<?php

namespace Drupal\lingotek_copy_source\Form;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\lingotek\Exception\LingotekApiException;
use Drupal\lingotek\Exception\LingotekDocumentArchivedException;
use Drupal\lingotek\Exception\LingotekDocumentLockedException;
use Drupal\lingotek\Exception\LingotekPaymentRequiredException;

class LingotekManagementFormBulkActions {

  /**
   * Request translations for a given content in a given language.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param string $langcode
   *   The language to download.
   */
  public static function requestTranslation(ContentEntityInterface $entity, $langcode, $job_id, &$context) {
    /** @var \Drupal\lingotek_copy_source\ProfileService $profileService */
    $profileService = \Drupal::service('lingotek_copy_source.profile_service');
    $lingotekConfiguration = \Drupal::service('lingotek.configuration');
    $languageLocaleMapper = \Drupal::service('lingotek.language_locale_mapper');
    $translationService = \Drupal::service('lingotek.content_translation');
    $entityTypeBundleInfo = \Drupal::service('entity_type.bundle.info');

    $context['message'] = t('Requesting translation for @type %label to language @language.', [
      '@type' => $entity->getEntityType()
        ->getLabel(),
      '%label' => $entity->label(),
      '@language' => $langcode,
    ]);
    $locale = $languageLocaleMapper->getLocaleForLangcode($langcode);
    if ($profile = $lingotekConfiguration->getEntityProfile($entity, FALSE)) {
      if ($profileService->checkIfCopySource($profile, $langcode)) {
        $data = $translationService->getSourceData($entity);
        $translationService->saveTargetData($entity, $langcode, $data);
      }
      else {
        try {
          $translationService->addTarget($entity, $locale);
        }
        catch (LingotekPaymentRequiredException $exception) {
          \Drupal::messenger()
            ->addError(t('Community has been disabled. Please contact support@lingotek.com to re-enable your community.'));
        }
        catch (LingotekDocumentArchivedException $exception) {
          \Drupal::messenger()
            ->addError(t('Document @entity_type %title has been archived. Please upload again.', [
              '@entity_type' => $entity->getEntityTypeId(),
              '%title' => $entity->label(),
            ]));
        }
        catch (LingotekDocumentLockedException $exception) {
          \Drupal::messenger()
            ->addError(t('Document @entity_type %title has a new version. The document id has been updated for all future interactions. Please try again.', [
              '@entity_type' => $entity->getEntityTypeId(),
              '%title' => $entity->label(),
            ]));
        }
        catch (LingotekApiException $exception) {
          \Drupal::messenger()
            ->addError(t('The request for @entity_type %title translation failed. Please try again.', [
              '@entity_type' => $entity->getEntityTypeId(),
              '%title' => $entity->label(),
            ]));
        }
      }
    }
    else {
      $bundleInfos = $entityTypeBundleInfo->getBundleInfo($entity->getEntityTypeId());
      \Drupal::messenger()
        ->addWarning(t('The @type %label has no profile assigned so it was not processed.',
          [
            '@type' => $bundleInfos[$entity->bundle()]['label'],
            '%label' => $entity->label(),
          ]));
    }
  }

  /**
   * Request all translations for a given content.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   */
  public static function requestTranslations(ContentEntityInterface $entity, $language, $job_id, &$context) {
    /** @var \Drupal\lingotek_copy_source\ProfileService $profileService */
    $profileService = \Drupal::service('lingotek_copy_source.profile_service');
    $lingotekConfiguration = \Drupal::service('lingotek.configuration');
    $languageLocaleMapper = \Drupal::service('lingotek.language_locale_mapper');
    $languageManager = \Drupal::service('language_manager');
    $translationService = \Drupal::service('lingotek.content_translation');
    $entityTypeBundleInfo = \Drupal::service('entity_type.bundle.info');

    $result = NULL;
    $context['message'] = t('Requesting translations for @type %label.', [
      '@type' => $entity->getEntityType()
        ->getLabel(),
      '%label' => $entity->label(),
    ]);
    if ($profile = $lingotekConfiguration->getEntityProfile($entity, FALSE)) {
      $target_languages = $languageManager->getLanguages();
      $target_languages = array_filter($target_languages, function (LanguageInterface $language) use ($lingotekConfiguration) {
        $configLanguage = ConfigurableLanguage::load($language->getId());
        return $lingotekConfiguration->isLanguageEnabled($configLanguage);
      });
      $entity_langcode = $entity->getUntranslated()->language()->getId();
      foreach ($target_languages as $langcode => $language) {
        if ($langcode !== $entity_langcode) {
          if ($profileService->checkIfCopySource($profile, $langcode)) {
            $data = $translationService->getSourceData($entity);
            $translationService->saveTargetData($entity, $langcode, $data);
          }
          else {
            $locale = $languageLocaleMapper->getLocaleForLangcode($langcode);
            try {
              $translationService->addTarget($entity, $locale);
            }
            catch (LingotekPaymentRequiredException $exception) {
              \Drupal::messenger()
                ->addError(t('Community has been disabled. Please contact support@lingotek.com to re-enable your community.'));
            }
            catch (LingotekDocumentArchivedException $exception) {
              \Drupal::messenger()
                ->addError(t('Document @entity_type %title has been archived. Please upload again.', [
                  '@entity_type' => $entity->getEntityTypeId(),
                  '%title' => $entity->label(),
                ]));
            }
            catch (LingotekDocumentLockedException $exception) {
              \Drupal::messenger()
                ->addError(t('Document @entity_type %title has a new version. The document id has been updated for all future interactions. Please try again.', [
                  '@entity_type' => $entity->getEntityTypeId(),
                  '%title' => $entity->label(),
                ]));
            }
            catch (LingotekApiException $exception) {
              \Drupal::messenger()
                ->addError(t('The request for @entity_type %title translation failed. Please try again.', [
                  '@entity_type' => $entity->getEntityTypeId(),
                  '%title' => $entity->label(),
                ]));
            }
          }
        }
      }
    }

    else {
      $bundleInfos = $entityTypeBundleInfo->getBundleInfo($entity->getEntityTypeId());
      \Drupal::messenger()
        ->addWarning(t('The @type %label has no profile assigned so it was not processed.',
          [
            '@type' => $bundleInfos[$entity->bundle()]['label'],
            '%label' => $entity->label(),
          ]));
    }
    return $result;

  }

}
